@@ -116,6 +116,12 @@ class Context(object):
         if not totest.isdir():
             raise FileError("%s already exists" % totest)
         return totest
+        
+    def validate_path(self, path):
+        """Validates that the path is legitimate. It doesn't
+        need to point to anything existing, it just has to
+        be able to pass _normalize_path."""
+        return self._normalize_path(path)
     
 class SecureContext(Context):
     # this is not the ideal place for this
