@@ -288,7 +288,8 @@ class SVNNativeCommand(object):
     
 def add_std(parser, cl=True, depth=True, files=True,
     recursive=False, quiet=True, xml=False, revision=False,
-    verbose=False, force=False):
+    verbose=False, force=False, parents=False, message=False, 
+    accept=False):
     parser.add_argument("--help", "-h", action=_HelpAction,
                     default=argparse.SUPPRESS,
                     help="show this message and exit")
@@ -310,6 +311,10 @@ def add_std(parser, cl=True, depth=True, files=True,
     if verbose:
         parser.add_argument("--verbose", "-v", action="store_true",
             help="print extra information")
+
+    if message:
+        parser.add_argument("--message", "-m", 
+            help="specify log message")
             
     if cl:
         parser.add_argument("--changelist", "--cl",
@@ -337,6 +342,15 @@ def add_std(parser, cl=True, depth=True, files=True,
     if force:
         parser.add_argument("--force", action="store_true",
             help="force operation to run")
+            
+    if parents:
+        parser.add_argument("--parents", action="store_true",
+            help="make intermediate directories")
+            
+    if accept:
+        parser.add_argument("--accept", 
+            choices=('base', 'working', 'mine-full', 'theirs-full'),
+            help="specify automatic conflict resolution source")
     
 class SVN_revert(SVNNativeCommand):
     existing_files = ["files"]
@@ -466,11 +480,9 @@ class SVN_commit(SVNNativeCommand):
     
     parser = OptionParser(prog="svn commit",
         description="commit (ci): Send changes from your working copy to the repository.")
-    add_std(parser)
+    add_std(parser, message=True)
     parser.add_argument("--no-unlock", action="store_true",
         help="don't unlock the targets")
-    parser.add_argument("--message", "-m", 
-        help="specify log message")
     parser.add_argument("--with-revprop",
         help="set revision property ARG in new revision using the name[=value] format")
     parser.add_argument("--keep-changelists", action="store_true",
@@ -558,6 +570,8 @@ class SVN_info(SVNNativeCommand):
 class SVN_blame(SVNNativeCommand):
     existing_files = ["files"]
     
+    aliases = ["praise", "annotate", "ann"]
+    
     reads_remote = True
     writes_remote = False
     
@@ -621,22 +635,41 @@ class SVN_resolve(SVNNativeCommand):
     parser = OptionParser(prog="svn resolve",
         description="Resolve conflicts on working copy files or directories. (--accept is required)")
     
-    add_std(parser)
-    parser.add_argument("--accept", 
-        choices=('base', 'working', 'mine-full', 'theirs-full'),
-        help="specify automatic conflict resolution source")
+    add_std(parser, accept=True)
         
 def _check_url(url_string):
+    if " " in url_string:
+        raise FileError("Spaces not allowed in URLs: %s" % (url_string))
     url = urlsplit(url_string)
     scheme = url[0].lower()
     if scheme not in ["http", "https", "svn+ssh"]:
         raise FileError("Unsupported URL: %s" % (url_string))
     
 
+def _validate_urls_and_files(context, values):
+    if len(values.src) == 1 and values.src[0].find("://") > -1:
+        _check_url(values.src[0])
+        if values.dst.find("://") > -1:
+            _check_url(values.dst)
+        else:
+            context.validate_new_file_or_existing_dir(values.dst)
+        return
+        
+    for f in values.src:
+        context.validate_existing_file(f)
+    
+    if len(values.src) == 1:
+        context.validate_new_file_or_existing_dir(values.dst)
+    else:
+        context.validate_existing_directory(values.dst)
+    
+
 class SVN_copy(SVNNativeCommand):
     reads_remote = True
     writes_remote = True
     
+    aliases = ["cp"]
+    
     parser = OptionParser(prog="svn copy",
         description="""copy (cp): Duplicate something in working copy or repository, remembering history.
 
@@ -657,11 +690,7 @@ to propagate merge tracking information from the source of the copy
 to the destination.
 """)
     add_std(parser, files=False, revision=True, depth=False,
-            cl=False)
-    parser.add_argument("--parents", action="store_true",
-        help="make intermediate directories")
-    parser.add_argument("--message", "-m",
-        help="specify log message MESSAGE")
+            cl=False, parents=True, message=True)
     parser.add_argument("--with-revprop",
         help="set revision property WITH_REVPROP in new revision using the name[=value] format")
     
@@ -669,23 +698,392 @@ to the destination.
     parser.add_argument("dst", help="destination file, url or directory")
     
     def validate_args(self, context, values):
-        if len(values.src) == 1 and values.src[0].find("://") > -1:
-            _check_url(values.src[0])
-            if values.dst.find("://") > -1:
-                _check_url(values.dst)
-            else:
-                context.validate_new_file_or_existing_dir(values.dst)
-            return
-            
-        for f in values.src:
-            context.validate_existing_file(f)
-        
-        if len(values.src) == 1:
-            context.validate_new_file_or_existing_dir(values.dst)
-        else:
-            context.validate_existing_directory(values.dst)
-        
+        _validate_urls_and_files(context, values)
     
+class SVN_move(SVNNativeCommand):
+    reads_remote = True
+    writes_remote = True
+    
+    aliases = ["mv", "rename", "ren"]
+    
+    parser = OptionParser(prog="svn move",
+        description="""Move and/or rename something in working copy or repository.
+
+When moving multiple sources, they will be added as children of DST,
+which must be a directory.
+
+  Note:  this subcommand is equivalent to a 'copy' and 'delete'.
+  Note:  the --revision option has no use and is deprecated.
+
+  SRC and DST can both be working copy (WC) paths or URLs:
+    WC  -> WC:   move and schedule for addition (with history)
+    URL -> URL:  complete server-side rename.
+  All the SRCs must be of the same type.
+""")
+
+    add_std(parser, files=False, depth=False, force=True,
+            cl=False, parents=True, message=True)
+    parser.add_argument("--with-revprop",
+        help="set revision property WITH_REVPROP in new revision using the name[=value] format")
+    parser.add_argument("src", nargs="+", help="source file(s) or URL(s)")
+    parser.add_argument("dst", help="destination file, url or directory")
+
+    def validate_args(self, context, values):
+        _validate_urls_and_files(context, values)
+    
+class SVN_mkdir(SVNNativeCommand):
+    reads_remote = True
+    writes_remote = True
+    
+    parser = OptionParser(prog="svn mkdir",
+        description="""Create a new directory under version control.
+
+  Create version controlled directories.
+
+  1. Each directory specified by a working copy PATH is created locally
+    and scheduled for addition upon the next commit.
+
+  2. Each directory specified by a URL is created in the repository via
+    an immediate commit.
+
+  In both cases, all the intermediate directories must already exist,
+  unless the --parents option is given.
+""")
+
+    add_std(parser, files=False, parents=True, depth=False,
+        message=True)
+    
+    parser.add_argument("path", nargs="+", help="URL or file path to create")
+    
+    def validate_args(self, context, values):
+        for v in values.path:
+            if v.find("://") > -1:
+                _check_url(v)
+            else:
+                context.validate_new_directory(v)
+    
+class SVN_merge(SVNNativeCommand):
+    reads_remote = True
+    writes_remote = True
+    
+    parser = OptionParser(prog="svn merge",
+        description="""Apply the differences between two sources to a working copy path.
+
+usage: 1. merge sourceURL1[@N] sourceURL2[@M] [WCPATH]
+       2. merge sourceWCPATH1@N sourceWCPATH2@M [WCPATH]
+       3. merge [-c M[,N...] | -r N:M ...] SOURCE[@REV] [WCPATH]
+
+  1. In the first form, the source URLs are specified at revisions
+     N and M.  These are the two sources to be compared.  The revisions
+     default to HEAD if omitted.
+
+  2. In the second form, the URLs corresponding to the source working
+     copy paths define the sources to be compared.  The revisions must
+     be specified.
+
+  3. In the third form, SOURCE can be either a URL or a working copy
+     path (in which case its corresponding URL is used).  SOURCE (in
+     revision REV) is compared as it existed between revisions N and M
+     for each revision range provided.  If REV is not specified, HEAD
+     is assumed.  '-c M' is equivalent to '-r <M-1>:M', and '-c -M'
+     does the reverse: '-r M:<M-1>'.  If no revision ranges are
+     specified, the default range of 0:REV is used.  Multiple '-c'
+     and/or '-r' instances may be specified, and mixing of forward
+     and reverse ranges is allowed.
+
+  WCPATH is the working copy path that will receive the changes.
+  If WCPATH is omitted, a default value of '.' is assumed, unless
+  the sources have identical basenames that match a file within '.':
+  in which case, the differences will be applied to that file.
+
+  NOTE:  Subversion will only record metadata to track the merge
+  if the two sources are on the same line of history -- if the
+  first source is an ancestor of the second, or vice-versa.  This is
+  guaranteed to be the case when using the third form listed above.
+  The --ignore-ancestry option overrides this, forcing Subversion to
+  regard the sources as unrelated and not to track the merge.
+
+""")
+    add_std(parser, files=False, revision=True, force=True, accept=True)
+    
+    parser.add_argument("--change", "-c", 
+        help="the change made by revision ARG (like -r ARG-1:ARG) If ARG is negative this is like -r ARG:ARG-1")
+    parser.add_argument("--dry-run", action="store_true",
+        help="try operation but make no changes")
+    parser.add_argument("--record-only", action="store_true",
+        help="mark revisions as merged (use with -r)")
+    parser.add_argument("--ignore-ancestry", 
+        help="ignore ancestry when calculating merges")
+    parser.add_argument("--reintegrate", action="store_true",
+        help="lump-merge all of source URL's unmerged changes")
+    
+    parser.add_argument("paths", nargs="+")
+    
+    def validate_args(self, context, values):
+        if values.change:
+            if len(values.paths) > 2:
+                raise SVNError("When using -c, only SOURCE[@REV] [WCPATH] arguments are permitted.")
+                
+            if "://" in values.paths[0]:
+                _check_url(values.paths[0])
+            else:
+                context.validate_exists(values.paths[0])
+            
+            if len(values.paths) == 2:
+                if "://" in values.paths[1]:
+                    raise SVNError("The final argument must be a working copy path.")
+                context.validate_exists(values.paths[1])
+        else:
+            if len(values.paths) < 2 or len(values.paths) > 3:
+                raise SVNError("Two sources and an optional working copy path are required")
+            if "://" in values.paths[0]:
+                if "://" not in values.paths[1]:
+                    raise SVNError("Both sources must be URLs")
+                _check_url(values.paths[0])
+                _check_url(values.paths[1])
+            else:
+                if "://" in values.paths[1]:
+                    raise SVNError("Both sources must be working copy paths")
+                context.validate_exists(values.paths[0])
+                context.validate_exists(values.paths[1])
+            
+            if len(values.paths) == 3:
+                context.validate_exists(values.paths[2])
+     
+class SVN_switch(SVNNativeCommand):
+    reads_remote = True
+    writes_remote = True
+    
+    aliases = ["sw"]
+    
+    parser = OptionParser(prog="svn switch",
+        description="""Update the working copy to a different URL.
+
+usage: 1. switch URL[@PEGREV] [PATH]
+       2. switch --relocate FROM TO [PATH...]
+
+  1. Update the working copy to mirror a new URL within the repository.
+     This behaviour is similar to 'svn update', and is the way to
+     move a working copy to a branch or tag within the same repository.
+     If specified, PEGREV determines in which revision the target is first
+     looked up.
+
+     If --force is used, unversioned obstructing paths in the working
+     copy do not automatically cause a failure if the switch attempts to
+     add the same path.  If the obstructing path is the same type (file
+     or directory) as the corresponding path in the repository it becomes
+     versioned but its contents are left 'as-is' in the working copy.
+     This means that an obstructing directory's unversioned children may
+     also obstruct and become versioned.  For files, any content differences
+     between the obstruction and the repository are treated like a local
+     modification to the working copy.  All properties from the repository
+     are applied to the obstructing path.
+
+     Use the --set-depth option to set a new working copy depth on the
+     targets of this operation.  Currently, the depth of a working copy
+     directory can only be increased (telescoped more deeply); you cannot
+     make a directory more shallow.
+
+  2. Rewrite working copy URL metadata to reflect a syntactic change only.
+     This is used when repository's root URL changes (such as a scheme
+     or hostname change) but your working copy still reflects the same
+     directory within the same repository.
+""")
+    add_std(parser, files=False, cl=False, revision=True, force=True, accept=True)
+    
+    parser.add_argument("--relocate", metavar="FROM",
+        help="relocate via URL-rewriting")
+    parser.add_argument("--ignore-externals", action="store_true",
+        help="ignore externals definitions")
+    
+    parser.add_argument("url")
+    parser.add_argument("path", nargs="?")
+    
+    def validate_args(self, context, values):
+        if values.relocate:
+            _check_url(values.relocate)
+        
+        _check_url(values.url)
+        
+        if values.path:
+            context.validate_existing_directory(values.path)
+
+class SVN_delete(SVNNativeCommand):
+    reads_remote = True
+    writes_remote = True
+    
+    aliases = ["del", "remove", "rm"]
+    
+    parser = OptionParser(prog="svn delete",
+        description="""Remove files and directories from version control.
+
+usage: 1. delete PATH...
+       2. delete URL...
+
+  1. Each item specified by a PATH is scheduled for deletion upon
+    the next commit.  Files, and directories that have not been
+    committed, are immediately removed from the working copy
+    unless the --keep-local option is given.
+    PATHs that are, or contain, unversioned or modified items will
+    not be removed unless the --force option is given.
+
+  2. Each item specified by a URL is deleted from the repository
+    via an immediate commit.
+""")
+    add_std(parser, files=False, force=True, message=True, cl=False, 
+        depth=False)
+    parser.add_argument("--keep-local", action="store_true",
+        help="keep path in working copy")
+    parser.add_argument("files", nargs="+", metavar="PATH_OR_URL")
+    
+    def validate_args(self, context, values):
+        for v in values.files:
+            if "://" in v:
+                _check_url(v)
+            else:
+                context.validate_exists(v)
+            
+class SVN_cat(SVNNativeCommand):
+    reads_remote = True
+    writes_remote = False
+    
+    parser = OptionParser(prog="svn cat",
+        description="""Output the content of specified files or URLs.
+
+usage: cat TARGET[@REV]...
+
+  If specified, REV determines in which revision the target is first
+  looked up.
+""")
+    add_std(parser, revision=True, files=False, depth=False, quiet=False,
+                cl=False)
+    parser.add_argument("targets", nargs="+")
+    
+    def validate_args(self, context, values):
+        for v in values.targets:
+            if "://" in v:
+                _check_url(v)
+            else:
+                context.validate_path(v)
+
+class SVN_propset(SVNNativeCommand):
+    existing_files = ["path"]
+    
+    reads_remote = False
+    writes_remote = False
+    
+    aliases = ["pset", "ps"]
+    
+    parser = OptionParser(prog="svn propset",
+        description="""Set the value of a property on files, dirs, or revisions.
+
+  Note: svn recognizes the following special versioned properties
+  but will store any arbitrary properties set:
+    svn:ignore     - A newline separated list of file patterns to ignore.
+    svn:keywords   - Keywords to be expanded.  Valid keywords are:
+      URL, HeadURL             - The URL for the head version of the object.
+      Author, LastChangedBy    - The last person to modify the file.
+      Date, LastChangedDate    - The date/time the object was last modified.
+      Rev, Revision,           - The last revision the object changed.
+      LastChangedRevision
+      Id                       - A compressed summary of the previous
+                                   4 keywords.
+    svn:executable - If present, make the file executable.  Use
+      'svn propdel svn:executable PATH...' to clear.
+    svn:eol-style  - One of 'native', 'LF', 'CR', 'CRLF'.
+    svn:mime-type  - The mimetype of the file.  Used to determine
+      whether to merge the file, and how to serve it from Apache.
+      A mimetype beginning with 'text/' (or an absent mimetype) is
+      treated as text.  Anything else is treated as binary.
+    svn:externals  - A newline separated list of module specifiers,
+      each of which consists of a relative directory path, optional
+      revision flags and an URL.  The ordering of the three elements
+      implements different behavior.  Subversion 1.4 and earlier only
+      support the following formats and the URLs cannot have peg
+      revisions:
+        foo             http://example.com/repos/zig
+        foo/bar -r 1234 http://example.com/repos/zag
+      Subversion 1.5 and greater support the above formats and the
+      following formats where the URLs may have peg revisions:
+                http://example.com/repos/zig foo
+        -r 1234 http://example.com/repos/zig foo/bar
+      Relative URLs are supported in Subversion 1.5 and greater for
+      all above formats and are indicated by starting the URL with one
+      of the following strings
+        ../  to the parent directory of the extracted external
+        ^/   to the repository root
+        //   to the scheme
+        /    to the server root
+      The ambiguous format 'relative_path relative_path' is taken as
+      'relative_url relative_path' with peg revision support.
+    svn:needs-lock - If present, indicates that the file should be locked
+      before it is modified.  Makes the working copy file read-only
+      when it is not locked.  Use 'svn propdel svn:needs-lock PATH...'
+      to clear.
+
+  The svn:keywords, svn:executable, svn:eol-style, svn:mime-type and
+  svn:needs-lock properties cannot be set on a directory.  A non-recursive
+  attempt will fail, and a recursive attempt will set the property
+  only on the file children of the directory.
+""")
+    parser.add_argument("propname")
+    parser.add_argument("propval")
+    parser.add_argument("path", nargs="+")
+    add_std(parser, force=True, files=False)
+    
+class SVN_propget(SVNNativeCommand):
+    existing_files = ["files"]
+    
+    reads_remote = False
+    writes_remote = False
+    
+    aliases = ["pg", "pget"]
+    
+    parser = OptionParser(prog="svn propget",
+        description="""Print the value of a property on files, dirs, or revisions.
+
+  By default, this subcommand will add an extra newline to the end
+  of the property values so that the output looks pretty.  Also,
+  whenever there are multiple paths involved, each property value
+  is prefixed with the path with which it is associated.  Use
+  the --strict option to disable these beautifications (useful,
+  for example, when redirecting binary property values to a file).
+""")
+    parser.add_argument("propname")
+    add_std(parser, revision=True, quiet=False)
+    parser.add_argument("--strict", action="store_true",
+        help="use strict semantics")
+    parser.add_argument("--xml", action="store_true",
+        help="output in XML")
+    
+class SVN_propdel(SVNNativeCommand):
+    existing_files = ["files"]
+    
+    reads_remote = False
+    writes_remote = False
+    
+    aliases = ["pdel", "pd"]
+    
+    parser = OptionParser(prog="svn propdel",
+        description="""Remove a property from files, dirs, or revisions.
+""")
+    
+    parser.add_argument("propname")
+    add_std(parser)
+    
+class SVN_proplist(SVNNativeCommand):
+    existing_files = ["files"]
+    
+    reads_remote = False
+    writes_remote = False
+    aliases = ["plist", "pl"]
+    
+    parser = OptionParser(prog="svn proplist",
+        description="""List all properties on files, dirs, or revisions.""")
+    
+    add_std(parser, verbose=True)
+    parser.add_argument("--xml", action="store_true",
+        help="output in XML")
     
 class SVN_help(SVNNativeCommand):
     reads_remote=False
