@@ -178,7 +178,44 @@ def test_svn_native_commands():
         T("copy http://blazingthings.com/awesome/ http://blazingthings.com/awesome/a2"),
         T("copy file://foo/bar/baz .", file_error=True),
         T("copy http://www.blazingthings.com/awesome/@658 ."),
-        T("copy http://www.blazingthings.com/awesome/@658 newdir")
+        T("copy http://www.blazingthings.com/awesome/@658 newdir"),
+        T("move foo.txt bar.txt"),
+        T("move http://www.blazingthings.com/awesome/blah http://www.blazingthings.com/awesome/bar"),
+        T("move foo http://www.blazingthings.com/awesome/bar", file_error=True),
+        T("mkdir testdir"),
+        T("merge http://foobar.baz/branch1 http://foobar.baz/branch2 ."),
+        T("merge http://foobar.baz/branch1 http://foobar.baz/branch2"),
+        T("merge -c 10 http://foobar.baz/branch1"),
+        T("merge http://foobar.baz/branch1 .", error=True),
+        T("merge -c 10 http://foobar.baz/branch1 http://foobar.baz/branch2 .",
+            error=True),
+        T("merge -c 10 http://foobar.baz/branch1 http://foobar.baz/branch2",
+            error=True),
+        T("merge branch1 http://foobar.baz/branch1", error=True),
+        T("switch http://foobar.baz/branch1"),
+        T("switch http://foobar.baz/branch1 ."),
+        T("switch http://foobar.baz/branch1 http://foobar.baz/branch2",
+            file_error=True),
+        T("switch .", file_error=True),
+        T("switch --relocate http://foobar.baz/branch1 http://foobar.baz/branch2 ."),
+        T("delete --keep-local foo.txt"),
+        T("cat http://foobar.baz/file1"),
+        T("cat http://foobar.baz/file1@329"),
+        T("cat foo.txt@329"),
+        T("cat file:///etc/passwd", file_error=True),
+        T("propset foo.txt", error=True),
+        T("propset svn:externals foo.txt", error=True),
+        T("propset svn:externals http://blah foo.txt"),
+        T("propset svn:externals http://blah DOESNTEXIST", 
+            file_error=True),
+        T("propget svn:ignore"),
+        T("propget svn:mime-type foo.txt"),
+        T("propget svn:mime-type DOESNOTEXIST", file_error=True),
+        T("propdel svn:mime-type foo.txt"),
+        T("propdel svn:mime-type DOESNOTEXIST", file_error=True),
+        T("proplist"),
+        T("proplist foo.txt"),
+        T("proplist DOESNOTEXIST", file_error=True)
     ]
     
     def test_one(t):
